#include "Condition.hh"
#include "ThreadError.hh"

#include <string.h>
#include <iostream>
using namespace std;

Condition::Condition() {
	int res=pthread_cond_init(&condition_,NULL);
	if(res!=0) {
		cerr << "Condition::Condition(): error (" 
			<< strerror(res) << "," << res 
			<< ") in creating condition..." << endl;
		throw ThreadError("can't create condition...");
	}
}

Condition::~Condition() {
	int res=pthread_cond_destroy(&condition_);
	if(res!=0) {
		cerr << "Condition::~Condition(): error (" 
			<< strerror(res) << "," << res 
			<< ") in destroying condition..." << endl;
	}
}

void Condition::wait(Mutex& mutex) {
	int res=pthread_cond_wait(&condition_,&mutex.mutex_);
	if(res!=0) {
		cerr << "Condition::wait(): error (" 
			<< strerror(res) << "," << res 
			<< ") in waiting on condition..." << endl;
		throw ThreadError("error waiting on condition...");
	}
}

void Condition::notify_one() {
	int res=pthread_cond_signal(&condition_);
	if(res!=0) {
		cerr << "Condition::notify_one(): error (" 
			<< strerror(res) << "," << res 
			<< ") in signaling on condition..." << endl;
		throw ThreadError("error signaling...");
	}
}

void Condition::notify_all() {
	int res=pthread_cond_broadcast(&condition_);
	if(res!=0) {
		cerr << "Condition::notify_all(): error (" 
			<< strerror(res) << "," << res 
			<< ") in broadcasting on condition..." << endl;
		throw ThreadError("error broadcasting...");
	}
}


