#include "Thread.hh"
#include "ThreadError.hh"
#include "Semaphore.hh"
#include "Mutex.hh"
#include "Guard.hh"

#include <iostream>
#include <ctime>
#include <vector>

using namespace std;

class Base: public Thread {
	bool stopped_;
	Mutex m_;
	int id_;
	Semaphore& s_;
public:
	Base(int id, Semaphore& s)
	: Thread(true),
	  stopped_(false),
	  id_(id),
	  s_(s)
	{}
	void stop() {
		Guard guard(m_);
		stopped_=true;
	}
	bool is_stopped() {
		Guard gurad(m_);
		return stopped_;
	}
	int get_id() const {
		return id_;
	}
	Semaphore& get_semaphore() const {
		return s_;
	}
};

class P:public Base {
public:
	P(int id, Semaphore& s)
	: Base(id,s)
	{}

	void run() {
		while(1) {
			if(is_stopped()){
				cerr << "P(" << get_id() << ") stopped..." << endl;
				return;
			}

			sleep(1);
			cerr << "P(" << get_id() << ") going to signal..." << endl;
			get_semaphore().signal();
		}
	}
};

class C:public Base {
public:
	C(int id, Semaphore& s)
	: Base(id,s)
	{}

	void run() {
		while(1) {
			if(is_stopped()) {
				cerr << "C(" << get_id() << ") stopped..." << endl;
				return;
			}

			sleep(1);
			get_semaphore().wait();
			cerr << "C(" << get_id() << ") awake..." << endl;
		}
	}
};

static const int PRODUCERS_COUNT=1;
static const int CONSUMERS_COUNT=1;

int main() {
	Semaphore sem(0);

	vector<P*> producers;
	vector<C*> consumers;
	
	//creating threads
	for(int i=0;i<PRODUCERS_COUNT;i++) {
		producers.push_back(new P(i,sem));
	}
	for(int i=0;i<CONSUMERS_COUNT;i++) {
		consumers.push_back(new C(i,sem));
	}
	
	//starting threads
	for(vector<C*>::iterator it=consumers.begin();
				it!=consumers.end();++it){
		(*it)->start();
	}
	for(vector<P*>::iterator it=producers.begin();
				it!=producers.end();++it){
		(*it)->start();
	}

	sleep(20);

	// stopping threads
	for(vector<P*>::iterator it=producers.begin();
				it!=producers.end();++it){
		(*it)->stop();
	}
	for(vector<C*>::iterator it=consumers.begin();
				it!=consumers.end();++it){
		(*it)->stop();
	}

	// joining producers	
	for(vector<P*>::iterator it=producers.begin();
				it!=producers.end();++it){
		cerr << "joining producers..." << endl;
		(*it)->join();
	}
	
	// signaling consumers
	for(int i=0;i<CONSUMERS_COUNT+PRODUCERS_COUNT+1;i++) {
		cerr << "signal..." << endl;
		sem.signal();
	}
	
	// joining consumers
	for(vector<C*>::iterator it=consumers.begin();
				it!=consumers.end();++it){
		cerr << "joining consumers..." << endl;
		(*it)->join();
	}
	
	// deleting threads
	for(vector<P*>::iterator it=producers.begin();
				it!=producers.end();++it){
		cerr << "deleting producers..." << endl;
		delete (*it);
	}
	for(vector<C*>::iterator it=consumers.begin();
				it!=consumers.end();++it){
		cerr << "deleting consumers..." << endl;
		delete (*it);
	}

	return 0;
};



